from django.db import models


class LcboProduct(models.Model):
    name = models.CharField(max_length=200)
    price_in_cents = models.IntegerField(default=0, null=True)
    primary_category = models.CharField(max_length=100, null=True)
    secondary_category = models.CharField(max_length=100, null=True)
    package_unit_type = models.CharField(max_length=20, null=True)
    package_unit_volume_in_milliliters = models.IntegerField(default=0, null=True)
    total_package_units = models.IntegerField(default=0, null=True)
    volume_in_milliliters = models.IntegerField(default=0, null=True)
    alcohol_content = models.IntegerField(default=0, null=True)
    price_per_liter_of_alcohol_in_cents = models.IntegerField(null=False)
    price_per_liter_in_cents = models.IntegerField(default=0, null=True)
    producer_name = models.CharField(max_length=255, null=True)
    image_thumb_url = models.CharField(max_length=255, null=True)
    image_url = models.CharField(max_length=255, null=True)
    is_discontinued = models.BooleanField()
    is_dead = models.BooleanField()
    limited_time_offer_ends_on = models.CharField(max_length=255, null=True)
    tasting_note = models.CharField(max_length=400, null=True)
    description = models.CharField(max_length=400, null=True)
    origin = models.CharField(max_length=255, null=True)
    product_no = models.IntegerField()



import requests
import simplejson as json
import models
import settings


def _get_dict_for_one_page_of_products(page):
    response = requests.get(settings.LCBO_PRODUCTS_API_URL + "?per_page=100&page=%d" % page)
    return response.json()


def _store_products_from_a_page(lcbo_data):
    for product in lcbo_data:
            lcbo_product = models.LcboProduct()  # Initialize our lcbo product object
            for field in models.LcboProduct._meta.fields:  # For every field on our model definition
                setattr(lcbo_product, field.name, product.get(field.name, None))  # Set the field with api data, or None if it does not exist
            if product.get('name', None):
                lcbo_product.save()


def store_all_lcbo_products():
    page = 0
    has_next_page = True
    while has_next_page:
        page += 1
        try:  # Thanks alot lcbo api for giving us malformed json.
            lcbo_data = _get_dict_for_one_page_of_products(page)
            _store_products_from_a_page(lcbo_data['result'])
        except json.scanner.JSONDecodeError:
            print "Page %s blew up with malformed json. Call the cops." % page
        if lcbo_data['pager']['is_final_page']:
            has_next_page = False



from django.conf import settings

# Allows the usage of a custom url set in your project's settings.py, 
# Will default to lcboapi.com/products if the setting does not exist.
LCBO_PRODUCTS_API_URL = getattr(settings, 'LCBO_PRODUCTS_API_URL', 'http://lcboapi.com/products')



from django.core.management.base import BaseCommand
from liquor import liquor_analysis


class Command(BaseCommand):
    help = 'Analyze data from the LCBO Api'

    def handle(self, *args, **options):
        liquor_analysis.print_booze_a_lytics()
        maximum_price = int(input("Maximum amount of money to spend (Dollars):"))
        alcohol_content = int(input("Minimum alcohol percentage (Percent):"))
        liquor_analysis.filter_max_price_min_alcohol(maximum_price, alcohol_content)



import models
from django.db.models import Min


def get_alcoholic_beverages():
    return models.LcboProduct.objects.filter(price_per_liter_of_alcohol_in_cents__gt=0, is_discontinued=False, is_dead=False, alcohol_content__gt=0)


def cheapest_price_per_liter_of_alcohol():
    return get_alcoholic_beverages().aggregate(most_effective_alcohol=Min('price_per_liter_of_alcohol_in_cents'))['most_effective_alcohol']


def get_best_value_alcohol():
    return get_alcoholic_beverages().get(price_per_liter_of_alcohol_in_cents=cheapest_price_per_liter_of_alcohol())


def filter_max_price_min_alcohol(price, alcohol_content):
    alcoholic_beverages = models.LcboProduct.objects.filter(alcohol_content__gte=alcohol_content*100, price_in_cents__lte=price*100, price_per_liter_of_alcohol_in_cents__gt=0, is_discontinued=False, is_dead=False, alcohol_content__gt=0).order_by('-price_in_cents')
    if alcoholic_beverages.count() <= 0:
        print "Nothing found for those parameters."
    else:
        for beverage in alcoholic_beverages:
            print "(%s) %s (%dmL) (%s%% alcohol) for $%s" % (beverage.primary_category, beverage.name, beverage.volume_in_milliliters, beverage.alcohol_content/100, beverage.price_in_cents/100.00)


def print_booze_a_lytics():
    print "===Booze-a-lytics==="
    print "%d beers in the system" % models.LcboProduct.objects.filter(primary_category="Beer").count()
    best_value_alcohol = get_best_value_alcohol()
    print "Most cost effective beverage to get drunk is %s (%s)" % (best_value_alcohol.name, best_value_alcohol.primary_category)
    number_of_bottles = 2500 / best_value_alcohol.price_in_cents
    print "For $25, you should be able to get %d (%s mL bottles) of %s" % (number_of_bottles, best_value_alcohol.volume_in_milliliters, best_value_alcohol.name)
    print "==============="



from django.core.management.base import BaseCommand
from liquor import liquor_horse


class Command(BaseCommand):
    help = 'Copies data from the LCBO Api to the database'

    def handle(self, *args, **options):
        print "Begin stealing booze..."
        liquor_horse.store_all_lcbo_products()
        print "Booze successfully stolen."



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import codecs
from os import path
from setuptools import setup


read = lambda filepath: codecs.open(filepath, 'r', 'utf-8').read()


README_PATH = path.abspath(path.join(path.dirname(__file__), 'readme.txt'))

setup(
    name = "liquor",
    version = "0.1",
    author = "Nahim Nasser",
    author_email = "nnasser@gmail.com",
    classifiers = [
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License', # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    description = 'A simple Django app to download LCBO Product Data.',
    packages=[
        'liquor', 
        'liquor.management',
        'liquor.management.commands',
    ],
    long_description= read(README_PATH),
    install_requires=[
        'Django>=1.4',
        "requests>=1.1.0",
        "simplejson>=3.1.2",
    ],
)